@@ -5,13 +5,13 @@
 #'
 #' @param X The covariates.
 #' @param Y The event time (may be negative).
-#' @param D The event type (0: censoring, 1: failure).
+#' @param D The event type (0: censored, 1: failure).
 #' @param failure.times A vector of event times to fit the survival curve at. If NULL, then all the observed
 #'  failure times are used. This speeds up forest estimation by constraining the event grid. Observed event
 #'  times are rounded down to the last sorted occurance less than or equal to the specified failure time.
 #'  The time points should be in increasing order. Default is NULL.
 #' @param num.trees Number of trees grown in the forest. Default is 1000.
-#' @param sample.weights (experimental) Weights given to an observation in prediction.
+#' @param sample.weights Weights given to an observation in prediction.
 #'                       If NULL, each observation is given the same weight. Default is NULL.
 #' @param clusters Vector of integers or factors specifying which cluster each observation corresponds to.
 #'  Default is NULL (ignored).
@@ -34,8 +34,7 @@
 #'                      Default is 15.
 #' @param honesty Whether to use honest splitting (i.e., sub-sample splitting). Default is TRUE.
 #'  For a detailed description of honesty, honesty.fraction, honesty.prune.leaves, and recommendations for
-#'  parameter tuning, see the grf
-#'  \href{https://grf-labs.github.io/grf/REFERENCE.html#honesty-honesty-fraction-honesty-prune-leaves}{algorithm reference}.
+#'  parameter tuning, see the grf algorithm reference.
 #' @param honesty.fraction The fraction of data that will be used for determining splits if honesty = TRUE. Corresponds
 #'                         to set J1 in the notation of the paper. Default is 0.5 (i.e. half of the data is used for
 #'                         determining splits).
@@ -44,16 +43,20 @@
 #'  tree is skipped and does not contribute to the estimate). Setting this to FALSE may improve performance on
 #'  small/marginally powered data, but requires more trees (note: tuning does not adjust the number of trees).
 #'  Only applies if honesty is enabled. Default is TRUE.
-#' @param alpha A tuning parameter that controls the maximum imbalance of a split. Default is 0.05
-#'  (meaning the count of failures on each side of a split has to be at least 5 \% of the total observation count in a node)
+#' @param alpha A tuning parameter that controls the maximum imbalance of a split. The number of failures in
+#'  each child has to be at least one or `alpha` times the number of samples in the parent node. Default is 0.05.
 #' @param compute.oob.predictions Whether OOB predictions on training set should be precomputed. Default is TRUE.
+#' @param prediction.type The type of estimate of the survival function, choices are "Kaplan-Meier" or "Nelson-Aalen".
+#' Only relevant if `compute.oob.predictions` is TRUE. Default is "Kaplan-Meier".
 #' @param num.threads Number of threads used in training. By default, the number of threads is set
 #'                    to the maximum hardware concurrency.
 #' @param seed The seed of the C++ random number generator.
 #'
-#' @return A trained survival_forest forest object. The attribute `failure.times` contains the unique failure
-#'  times in the data set.
+#' @return A trained survival_forest forest object.
 #'
+#' @references Cui, Yifan, Michael R. Kosorok, Erik Sverdrup, Stefan Wager, and Ruoqing Zhu.
+#'  "Estimating Heterogeneous Treatment Effects with Right-Censored Data via Causal Survival Forests."
+#'  arXiv preprint arXiv:2001.09887, 2020.
 #' @references Ishwaran, Hemant, Udaya B. Kogalur, Eugene H. Blackstone, and Michael S. Lauer.
 #'   "Random survival forests." The Annals of Applied Statistics 2.3 (2008): 841-860.
 #'
@@ -99,6 +102,13 @@
 #' s.pred.grid <- predict(s.forest.grid)
 #' matpoints(s.pred.grid$failure.times, t(s.pred.grid$predictions[1:5, ]),
 #'           type = "l", lty = 2)
+#'
+#' # Compute OOB concordance based on the mortality score in Ishwaran et al. (2008).
+#' s.pred.nelson.aalen <- predict(s.forest, prediction.type = "Nelson-Aalen")
+#' chf.score <- rowSums(-log(s.pred.nelson.aalen$predictions))
+#' if (require("survival", quietly = TRUE)) {
+#'  concordance(Surv(Y, D) ~ chf.score, reverse = TRUE)
+#' }
 #' }
 #'
 #' @export
@@ -115,6 +125,7 @@ survival_forest <- function(X, Y, D,
                             honesty.fraction = 0.5,
                             honesty.prune.leaves = TRUE,
                             alpha = 0.05,
+                            prediction.type = c("Kaplan-Meier", "Nelson-Aalen"),
                             compute.oob.predictions = TRUE,
                             num.threads = NULL,
                             seed = runif(1, 0, .Machine$integer.max)) {
@@ -122,18 +133,23 @@ survival_forest <- function(X, Y, D,
   validate_sample_weights(sample.weights, X)
   Y <- validate_observations(Y, X)
   D <- validate_observations(D, X)
-  if(!all(D %in% c(0, 1))) {
+  if (!all(D %in% c(0, 1))) {
     stop("The censor values can only be 0 or 1.")
   }
   clusters <- validate_clusters(clusters, X)
   samples.per.cluster <- validate_equalize_cluster_weights(equalize.cluster.weights, clusters, sample.weights)
   num.threads <- validate_num_threads(num.threads)
+  prediction.type <- match.arg(prediction.type)
+  if (prediction.type == "Kaplan-Meier") {
+    prediction.type <- 0
+  } else if (prediction.type == "Nelson-Aalen") {
+    prediction.type <- 1
+  }
 
   # Relabel the times to consecutive integers such that:
-  # if the failure time is less than the smallest failure time: set it to 0
-  # if the failure time is above the latter, but less than the second smallest failure time: set it to 1
+  # if the event time is less than the smallest failure time: set it to 0
+  # if the event time is above the latter, but less than the second smallest failure time: set it to 1
   # etc. Will range from 0 to num.failures.
-  # (Entry 0 is for time k < t1)
   if (is.null(failure.times)) {
     failure.times <- sort(unique(Y[D == 1]))
   }
@@ -151,6 +167,7 @@ survival_forest <- function(X, Y, D,
                honesty.prune.leaves = honesty.prune.leaves,
                alpha = alpha,
                num.failures = length(failure.times),
+               prediction.type = prediction.type,
                compute.oob.predictions = compute.oob.predictions,
                num.threads = num.threads,
                seed = seed)
@@ -166,14 +183,15 @@ survival_forest <- function(X, Y, D,
   forest[["equalize.cluster.weights"]] <- equalize.cluster.weights
   forest[["has.missing.values"]] <- has.missing.values
   forest[["failure.times"]] <- failure.times
+  forest[["prediction.type"]] <- prediction.type
 
   forest
 }
 
-#' Predict with a survival forest forest
+#' Predict with a survival forest
 #'
-#' Gets estimates of the conditional survival function S(t, x) using a trained survival forest (estimated using
-#' Kaplan-Meier).
+#' Gets estimates of the conditional survival function S(t, x) using a trained survival forest. The curve can be
+#' estimated by Kaplan-Meier, or Nelson-Aalen.
 #'
 #' @param object The trained forest.
 #' @param newdata Points at which predictions should be made. If NULL, makes out-of-bag
@@ -182,12 +200,16 @@ survival_forest <- function(X, Y, D,
 #'                that this matrix should have the number of columns as the training
 #'                matrix, and that the columns must appear in the same order.
 #' @param failure.times A vector of failure times to make predictions at. If NULL, then the
-#'  failure times used for training the forest is used. Default is NULL.
+#'  failure times used for training the forest is used. The time points should be in increasing order. Default is NULL.
+#' @param prediction.type The type of estimate of the survival function, choices are "Kaplan-Meier" or "Nelson-Aalen".
+#'  The default is the prediction.type used to train the forest.
 #' @param num.threads Number of threads used in training. If set to NULL, the software
 #'                    automatically selects an appropriate amount.
 #' @param ... Additional arguments (currently ignored).
 #'
-#' @return Vector of predictions.
+#' @return A list with elements `failure.times`: a vector of event times t for the survival curve,
+#'  and `predictions`: a matrix of survival curves. Each row is the survival curve for
+#'  sample X_i: predictions[i, j] = S(failure.times[j], X_i).
 #'
 #' @examples
 #' \donttest{
@@ -231,6 +253,13 @@ survival_forest <- function(X, Y, D,
 #' s.pred.grid <- predict(s.forest.grid)
 #' matpoints(s.pred.grid$failure.times, t(s.pred.grid$predictions[1:5, ]),
 #'           type = "l", lty = 2)
+#'
+#' # Compute OOB concordance based on the mortality score in Ishwaran et al. (2008).
+#' s.pred.nelson.aalen <- predict(s.forest, prediction.type = "Nelson-Aalen")
+#' chf.score <- rowSums(-log(s.pred.nelson.aalen$predictions))
+#' if (require("survival", quietly = TRUE)) {
+#'  concordance(Surv(Y, D) ~ chf.score, reverse = TRUE)
+#' }
 #' }
 #'
 #' @method predict survival_forest
@@ -238,7 +267,9 @@ survival_forest <- function(X, Y, D,
 predict.survival_forest <- function(object,
                                     newdata = NULL,
                                     failure.times = NULL,
+                                    prediction.type = c("Kaplan-Meier", "Nelson-Aalen"),
                                     num.threads = NULL, ...) {
+  num.threads <- validate_num_threads(num.threads)
   if (is.null(failure.times)) {
     failure.times <- object[["failure.times"]]
     Y.relabeled <- object[["Y.relabeled"]]
@@ -246,24 +277,35 @@ predict.survival_forest <- function(object,
     Y.relabeled <- findInterval(object[["Y.orig"]], failure.times)
   }
 
-  # If possible, use pre-computed predictions.
-  failure.times.orig <- object[["failure.times"]]
-  if (is.null(newdata) && identical(failure.times, failure.times.orig) && !is.null(object$predictions)) {
-    return(list(predictions = object$predictions, failure.times = failure.times))
+  default.prediction.type <- length(prediction.type) == 2
+  prediction.type <- match.arg(prediction.type)
+  if (default.prediction.type) {
+    prediction.type <- object[["prediction.type"]]
+  } else if (prediction.type == "Kaplan-Meier") {
+    prediction.type <- 0
+  } else if (prediction.type == "Nelson-Aalen") {
+    prediction.type <- 1
   }
 
-  num.threads <- validate_num_threads(num.threads)
+  # If possible, use pre-computed predictions.
+  failure.times.orig <- object[["failure.times"]]
+  prediction.type.orig <- object[["prediction.type"]]
+  if (is.null(newdata) && identical(failure.times, failure.times.orig)
+      && identical(prediction.type, prediction.type.orig) && !is.null(object$predictions)) {
+    return(list(predictions = object$predictions, failure.times = failure.times))
+  }
 
   forest.short <- object[-which(names(object) == "X.orig")]
   X <- object[["X.orig"]]
   train.data <- create_train_matrices(X,
                                       outcome = Y.relabeled,
-                                      sample.weights = object[["sample.weights"]],
-                                      censor = object[["D.orig"]])
+                                      censor = object[["D.orig"]],
+                                      sample.weights = object[["sample.weights"]])
 
   args <- list(forest.object = forest.short,
                num.threads = num.threads,
-               num.failures = length(failure.times))
+               num.failures = length(failure.times),
+               prediction.type = prediction.type)
 
   if (!is.null(newdata)) {
     validate_newdata(newdata, X, allow.na = TRUE)
